//|------------------------------------------------------------------+
//|                                                     CSOMNode.mqh |
//|                        Copyright 2011, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//|------------------------------------------------------------------+
#property copyright "Copyright 2011, MetaQuotes Software Corp."
#property link		"http://www.mql5.com"
#property version	"1.00"

//------------------------------------------------------------------	CSOMNode
class CSOMNode // CSOMNode class
{
protected:
	int	m_x1;
	int	m_y1;
	int	m_x2;
	int	m_y2;
	double m_x;
	double m_y;
	double m_weights[];
	int m_dimension;
public:
	CSOMNode(); // class constructor
	~CSOMNode(); // class destructor
	void InitNode(int x1, int y1, int x2, int y2); // node initialization
	double X() const { return(m_x); } // coordinates of the center
	double Y() const { return(m_y); }
	void GetCoordinates(int &x1,int &y1,int &x2,int &y2); // return node coordinates
	double GetWeight(int weight_index); // returns weight
	double CalculateDistance(double &vector[]); // returns squared distance from weight to vector
	void AdjustWeights(double &vector[],double learning_rate,double influence); // adjust weights
	void SetDimension(int dimension); // sets dimension of weights vector
};
//------------------------------------------------------------------	CSOMNode
void CSOMNode::CSOMNode() // class constructor
{
	m_x1=0; m_y1=0; m_x2=0; m_y2=0; m_x=0.0; m_y=0.0;
}
//------------------------------------------------------------------	InitNode
void CSOMNode::InitNode(int x1, int y1, int x2, int y2) // initialize node parameters
{
	m_x1=x1; m_y1=y1; m_x2=x2; m_y2=y2; // sets coordinates
	m_x=x1+MathAbs((x2-x1))/2; m_y=y1+MathAbs((y2-y1))/2; // calc center
	ArrayResize(m_weights,m_dimension); // prepare weights
	for(int i=0; i<m_dimension; i++) m_weights[i]=255.0*rand()/32768; // initialize weights
}
//------------------------------------------------------------------	~CSOMNode
void CSOMNode::~CSOMNode() // class destructor
{
	ArrayResize(m_weights,0); 
}
//------------------------------------------------------------------	GetWeight
double CSOMNode::GetWeight(int weight_index) // returns weight
{
	if(weight_index>=0 && weight_index<m_dimension) return(m_weights[weight_index]); else return(0);
}
//------------------------------------------------------------------	GetCoordinates
void CSOMNode::GetCoordinates(int &x1,int &y1,int &x2,int &y2) // return coordinates of the node
{
	x1=m_x1; y1=m_y1; x2=m_x2; y2=m_y2;
}
//------------------------------------------------------------------	CalculateDistance
double CSOMNode::CalculateDistance(double &vector[]) // returns squared distance
{
	double dist_sqr=0;
	for(int i=0; i<m_dimension; i++) dist_sqr=dist_sqr+(vector[i]-m_weights[i])*(vector[i]-m_weights[i]);
	return(dist_sqr);
}
//------------------------------------------------------------------	AdjustWeights
void CSOMNode::AdjustWeights(double &vector[],double learning_rate,double influence) // adjust weights
{
	for(int i=0; i<m_dimension; i++) m_weights[i]=m_weights[i]+learning_rate*influence*(vector[i]-m_weights[i]);
}
//------------------------------------------------------------------	SetDimension
void CSOMNode::SetDimension(int dimension)
{
	m_dimension=dimension;
}